/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFGui.h"
#include "TFWindowHandler.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


FlexArray 	*TFGui::s_windowList = NULL;
FlexArray 	*TFGui::s_deleteList = NULL;
TFWindowMain 	*TFGui::s_winMain = NULL;


/*
 *  constructor: initialize everything 
 */
TFGui::TFGui ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFGui\n");
	SanityCheck::bailout ((s_windowList), "s_windowList!=NULL (Gui already running?)", "TFGui::TFGui");
	SanityCheck::bailout ((s_deleteList), "s_deleteList!=NULL (Gui already running?)", "TFGui::TFGui");
	SanityCheck::bailout ((s_winMain), "s_winMain!=NULL (Gui already running?)", "TFGui::TFGui");

	TFGui::s_windowList = new FlexArray ();
	TFGui::s_deleteList = new FlexArray ();
	TFGui::s_winMain = new TFWindowMain (TFGui::s_windowList);
}


/*
 *  destructor: clean up 
 */
TFGui::~TFGui ()
{
	delete TFGui::s_windowList;
	delete TFGui::s_deleteList;
	delete TFGui::s_winMain;
	
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFGui\n");
}


/*
 *  addHeightField: set the Height Field we draw and create a window for it
 */
void TFGui::addHeightField (HeightField *HF)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFGui::addHeightField");

	TFWindowHandler 	*tfWinHandler;

	tfWinHandler = new TFWindowHandler (TFGui::s_windowList);
	tfWinHandler->setHeightField (HF);
	TFGui::s_windowList->append (tfWinHandler);
	connect_to_method (Gtk_Main::timeout(50), tfWinHandler, &TFWindowHandler::idleRedrawCallback);
}


/*
 *  scheduleDelete : add a height field to the s_deleteList
 */
void TFGui::scheduleDelete (HeightField *HF)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFGui::scheduleForCleanup");

	TFGui::s_deleteList->append (HF);
}


/*
 *  cleanup: delete all the windows pointed at by the HFs in s_deleteList. 
 * 	always returns 0 so that the gtk timeout will terminate. 
 */
int TFGui::cleanup ()
{
	TFWindowHandler		*TFWH;
	HeightField		*HF;
	int			i = 0, 
				j = 0,
				limDel = TFGui::s_deleteList->getSize (),
				limWin;

	SanityCheck::bailout ((!limDel), "limDel==0", "TFGui::cleanup");

	while (i < limDel)
		{
		HF = static_cast<HeightField*>(TFGui::s_deleteList->El(i));
		limWin = TFGui::s_windowList->getSize ();
		j = 0; 

		SanityCheck::bailout ((!HF), "HF==NULL", "TFGui::cleanup");
		SanityCheck::bailout ((!limWin), "limWin==0", "TFGui::cleanup");

		while (j < limWin)
			{
			TFWH = static_cast<TFWindowHandler*>(TFGui::s_windowList->El(j));
			if (HF == (TFWH->getHeightField()))
				break;
			j++;
			}

		if (j < limWin)
			{
			delete TFWH;
			TFGui::s_deleteList->remove (i);
			TFGui::s_windowList->remove (j);
			limDel--;
			}
		else
			i++;
		}

	return 0;			// force timeout connection to stop
}


